@@ -25,17 +25,19 @@
 
 #include "X11ImageGrabber.h"
 
-#include <QX11Info>
-#include <QStack>
+#include <KWindowSystem>
+
 #include <QDBusConnection>
-#include <QDBusInterface>
 #include <QDBusConnectionInterface>
-#include <QDBusReply>
-#include <QPainter>
-#include <QGraphicsScene>
-#include <QGraphicsPixmapItem>
+#include <QDBusInterface>
 #include <QGraphicsDropShadowEffect>
+#include <QGraphicsPixmapItem>
+#include <QGraphicsScene>
+#include <QPainter>
 #include <QSet>
+#include <QStack>
+#include <QX11Info>
+#include <QtMath>
 
 #include <KWindowSystem>
 #include <KWindowInfo>
@@ -44,6 +46,9 @@
 #include <xcb/xcb_util.h>
 #include <xcb/xfixes.h>
 
+#include <X11/Xatom.h>
+#include <X11/Xdefs.h>
+
 X11ImageGrabber::X11ImageGrabber(QObject *parent) :
     ImageGrabber(parent)
 {
@@ -318,6 +323,21 @@ QPixmap X11ImageGrabber::getToplevelPixmap(QRect rect, bool blendPointer)
     // Treat a null rect as an alias for capturing fullscreen
     if (!rect.isValid()) {
         rect = getDrawableGeometry(rootWindow);
+    } else {
+        QRegion screenRegion;
+        for (auto screen : QGuiApplication::screens()) {
+            QRect screenRect = screen->geometry();
+
+            // Do not use setSize() here, because QSize::operator*=()
+            // performs qRound() which can result in xcb_image_get() failing
+            const qreal dpr = screen->devicePixelRatio();
+            screenRect.setHeight(qFloor(screenRect.height() * dpr));
+            screenRect.setWidth(qFloor(screenRect.width() * dpr));
+
+            screenRegion += screenRect;
+        }
+
+        rect = (screenRegion & rect).boundingRect();
     }
 
     QPixmap nativePixmap = getPixmapFromDrawable(rootWindow, rect);
@@ -364,7 +384,7 @@ bool X11ImageGrabber::isKWinAvailable()
 {
     if (QDBusConnection::sessionBus().interface()->isServiceRegistered(QStringLiteral("org.kde.KWin"))) {
         QDBusInterface interface(QStringLiteral("org.kde.KWin"), QStringLiteral("/Effects"), QStringLiteral("org.kde.kwin.Effects"));
-        QDBusReply<bool> reply = interface.call(QStringLiteral("isEffectLoaded"), "screenshot");
+        QDBusReply<bool> reply = interface.call(QStringLiteral("isEffectLoaded"), QStringLiteral("screenshot"));
 
         return reply.value();
     }
@@ -413,6 +433,12 @@ void X11ImageGrabber::rectangleSelectionConfirmed(const QPixmap &pixmap, const Q
 
 // grabber methods
 
+void X11ImageGrabber::updateWindowTitle(xcb_window_t window)
+{
+    QString windowTitle = KWindowSystem::readNameProperty(window, XA_WM_NAME);
+    emit windowTitleChanged(windowTitle);
+}
+
 void X11ImageGrabber::grabFullScreen()
 {
     mPixmap = getToplevelPixmap(QRect(), mCapturePointer);
@@ -421,7 +447,8 @@ void X11ImageGrabber::grabFullScreen()
 
 void X11ImageGrabber::grabTransientWithParent()
 {
-    xcb_window_t curWin = getRealWindowUnderCursor();
+	xcb_window_t curWin = getRealWindowUnderCursor();
+    updateWindowTitle(curWin);
 
     // grab the image early
 
@@ -433,13 +460,20 @@ void X11ImageGrabber::grabTransientWithParent()
 
     QSet<xcb_window_t> transients;
     xcb_window_t parentWinId = curWin;
+    const QRect desktopRect(0, 0, 1, 1);
     do {
         // find parent window and add the window to the visible region
         xcb_window_t winId = parentWinId;
         QRect winRect;
         parentWinId = getTransientWindowParent(winId, winRect);
         transients << winId;
-        clipRegion += winRect;
+
+        // Don't include the 1x1 pixel sized desktop window in the top left corner that is present
+        // if the window is a QDialog without a parent.
+        // BUG: 376350
+        if (winRect != desktopRect) {
+            clipRegion += winRect;
+        }
 
         // Continue walking only if this is a transient window (having a parent)
     } while (parentWinId != XCB_WINDOW_NONE && !transients.contains(parentWinId));
@@ -507,6 +541,7 @@ void X11ImageGrabber::grabTransientWithParent()
 void X11ImageGrabber::grabActiveWindow()
 {
     xcb_window_t activeWindow = KWindowSystem::activeWindow();
+    updateWindowTitle(activeWindow);
 
     // if KWin is available, use the KWin DBus interfaces
 
@@ -535,6 +570,9 @@ void X11ImageGrabber::grabActiveWindow()
 
 void X11ImageGrabber::grabWindowUnderCursor()
 {
+    const xcb_window_t windowUnderCursor = getRealWindowUnderCursor();
+    updateWindowTitle(windowUnderCursor);
+
     // if KWin is available, use the KWin DBus interfaces
 
     if (mCaptureDecorations && isKWinAvailable()) {
@@ -557,7 +595,7 @@ void X11ImageGrabber::grabWindowUnderCursor()
 
     // else, go native
 
-    return grabApplicationWindowHelper(getRealWindowUnderCursor());
+    return grabApplicationWindowHelper(windowUnderCursor);
 }
 
 void X11ImageGrabber::grabApplicationWindowHelper(xcb_window_t window)
@@ -599,7 +637,9 @@ QRect X11ImageGrabber::getDrawableGeometry(xcb_drawable_t drawable)
 
     xcb_get_geometry_cookie_t geomCookie = xcb_get_geometry_unchecked(xcbConn, drawable);
     CScopedPointer<xcb_get_geometry_reply_t> geomReply(xcb_get_geometry_reply(xcbConn, geomCookie, NULL));
-
+    if (geomReply.isNull()) {
+        return QRect();
+    }
     return QRect(geomReply->x, geomReply->y, geomReply->width, geomReply->height);
 }
 
