@@ -19,8 +19,6 @@
 
 #include "SpectacleConfig.h"
 
-#include <QStandardPaths>
-
 SpectacleConfig::SpectacleConfig(QObject *parent) :
     QObject(parent)
 {
@@ -43,7 +41,7 @@ SpectacleConfig* SpectacleConfig::instance()
 QUrl SpectacleConfig::lastSaveAsLocation() const
 {
     return mGeneralConfig.readEntry(QStringLiteral("lastSaveAsLocation"),
-                                    QUrl::fromUserInput(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation)));
+                                    QUrl::fromUserInput(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation) + QStringLiteral("/")));
 }
 
 void SpectacleConfig::setLastSaveAsLocation(const QUrl &location)
@@ -117,6 +115,19 @@ void SpectacleConfig::setCaptureTransientWindowOnlyChecked(bool enabled)
     mGuiConfig.sync();
 }
 
+// quit after saving, copying, or exporting the image
+
+bool SpectacleConfig::quitAfterSaveOrCopyChecked() const
+{
+    return mGuiConfig.readEntry(QStringLiteral("quitAfterSaveCopyExport"), false);
+}
+
+void SpectacleConfig::setQuitAfterSaveOrCopyChecked(bool enabled)
+{
+    mGuiConfig.writeEntry(QStringLiteral("quitAfterSaveCopyExport"), enabled);
+    mGuiConfig.sync();
+}
+
 // capture delay
 
 qreal SpectacleConfig::captureDelay() const
@@ -143,19 +154,6 @@ void SpectacleConfig::setCaptureMode(int index)
     mGuiConfig.sync();
 }
 
-// dynamic save button
-
-bool SpectacleConfig::useDynamicSaveButton() const
-{
-    return mGuiConfig.readEntry(QStringLiteral("dynamicSaveButton"), false);
-}
-
-void SpectacleConfig::setUseDynamicSaveButton(bool enabled)
-{
-    mGuiConfig.writeEntry(QStringLiteral("dynamicSaveButton"), enabled);
-    mGuiConfig.sync();
-}
-
 // remember last rectangular region
 
 bool SpectacleConfig::rememberLastRectangularRegion() const
@@ -184,14 +182,20 @@ void SpectacleConfig::setUseLightRegionMaskColour(bool enabled)
 
 // last used save mode
 
-int SpectacleConfig::lastUsedSaveMode() const
+SaveMode SpectacleConfig::lastUsedSaveMode() const
 {
-    return mGuiConfig.readEntry(QStringLiteral("lastUsedSaveMode"), 0);
+    switch (mGuiConfig.readEntry(QStringLiteral("lastUsedSaveMode"), 0)) {
+        default:
+        case 0:
+            return SaveMode::SaveAs;
+        case 1:
+            return SaveMode::Save;
+    }
 }
 
-void SpectacleConfig::setLastUsedSaveMode(int index)
+void SpectacleConfig::setLastUsedSaveMode(SaveMode mode)
 {
-    mGuiConfig.writeEntry(QStringLiteral("lastUsedSaveMode"), index);
+    mGuiConfig.writeEntry(QStringLiteral("lastUsedSaveMode"), static_cast<int>(mode));
     mGuiConfig.sync();
 }
 
@@ -235,3 +239,17 @@ void SpectacleConfig::setCopySaveLocationToClipboard(bool enabled)
     mGeneralConfig.writeEntry(QStringLiteral("copySaveLocation"), enabled);
     mGeneralConfig.sync();
 }
+
+// autosave image format
+
+QString SpectacleConfig::saveImageFormat() const
+{
+    return mGeneralConfig.readEntry(QStringLiteral("default-save-image-format"),
+                             QStringLiteral("png"));
+}
+
+void SpectacleConfig::setSaveImageFormat(const QString &saveFmt)
+{
+    mGeneralConfig.writeEntry(QStringLiteral("default-save-image-format"), saveFmt);
+    mGeneralConfig.sync();
+}
